cpdef bint is_alpha(unicode string)
cpdef bint is_digit(unicode string)
cpdef bint is_punct(unicode string)
cpdef bint is_space(unicode string)
cpdef bint is_ascii(unicode string)
cpdef bint is_title(unicode string)
cpdef bint is_lower(unicode string)
cpdef bint is_upper(unicode string)
cpdef bint like_email(unicode string)
cpdef bint like_url(unicode string)
cpdef bint like_number(unicode string)
cpdef unicode word_shape(unicode string)
